#if !defined(_GENSTAT_H_INCLUDED)
#define _GENSTAT_H_INCLUDED

/*
**  Copyright (c) 1997 Pervasive Software Inc.  All rights reserved.
**
**  genstat.h
**
**  Brendan B. Boerner
**  7/14/97
**
**  Pervasive Software Products status codes.
**
**  Status code equates
**
**     Code range:
**      -100  - -199  ==>  Informative codes
**         1  -  199  ==>  Btrieve Status Codes
**       200  -  399  ==>  XQLP codes
**       400  -  499  ==>  XQLI codes
**       500  -  799  ==>  SQL syntax errors
**       800  -  999  ==>  XQLM execution errors
**      1000  - 1999  ==>  Btrieve Status Codes for OS2
**      2000  - 2099  ==>  Btrieve Interface
**      2100  - 2199  ==>  Scalable SQL Requester status codes
**      2200  - 2299  ==>  Environment specific Scalable SQL status codes
**      2300  - 2399  ==>  Named Database support, etc.
**      2400  - 2499  ==>  Microkernel Interface.
**      2500  - 2699  ==>  Embedded SQL
**      2700  - 2799  ==>  DAL Server
**      2800  - 2899  ==>  SSQL Database Router and EDA Adapter
**      2900  - 2999  ==>  Inscribe engine
**      4000  - 4999  ==>  DBU error codes
**      5000  - 5199  ==>  Internal error codes
**      6000  - ????  ==>  DBS error codes
**      7000  - 7099  ==>  License Manager error codes
**      8000  - 8499  ==>  SCM/AOSS/EventLog error codes
**      8500  - 8589  ==>  ECASAPI error codes
**      8590  - 8599  ==>  W3DBSMGR error codes
**
*/


#define E_FATAL         -1
#define E_NONE          0


/*
**  SCM/AOSS/EventLog error codes
*/

/* E_SCM_MEM: error allocating memory */
#define E_SCM_MEM               8001

/* E_SCM_CID_NOTFOUND: Component ID string not found. */
#define E_SCM_CID_NOTFOUND      8002

/* 
**  E_SCM_W16W32_GPAKERNEL: W16 cannot get address W32 APIs:
**  CallProc32W(), FreeLibrary32W(), GetProcAddress32W()
**  GetVDMPointer32W() or LoadLibraryEx32W
*/
#define E_SCM_W16W32_GPAKERNEL  8003

/* E_SCM_INV_PLATCALLTYPE: unsupported platform/call type */
#define E_SCM_INV_PLATCALLTYPE  8004

/* E_SCM_API_NOTINITED: API not initialized */
#define E_SCM_API_NOTINITED     8005

/*
**  E_SCM_MOD_NOTFOUND: cannot operate on module, module not found
**  in ModDir or OS path.
*/
#define E_SCM_MOD_NOTFOUND      8006

/* E_SCM_W32VERS: error retrieving Win32 version information */
#define E_SCM_W32VERS           8007

/*
**  E_SCM_DUP_NOTALLOWED: duplicate platforms, types and versions
**  not allowed.
*/
#define E_SCM_DUP_NOTALLOWED    8008

/*
**  E_SCM_CID_MISSING_REQEDINFO: Component ID string is missing required
**  information (platform, type, major/minor functional level).
*/
#define E_SCM_CID_MISSING_REQEDINFO 8009

/* E_SCM_FILE_OPEN: Error opening a file. */
#define E_SCM_FILE_OPEN         8010

/* E_SCM_SYM_EXPORT: Error defining export (e.g. #defineDLLEntry()). */
#define E_SCM_SYM_EXPORT        8011

/*
**  E_SCM_MISSING_REQEDINFO: Required information is missing (platform
**  type, major/minor functional level).
*/
#define E_SCM_MISSING_REQEDINFO 8012

/* E_SCM_OPEN_OSDIR: Error locating component. */
#define E_SCM_OPEN_OSDIR        8013

/* E_SCM_INFOBASE_READ: Error reading infobase. */
#define E_SCM_INFOBASE_READ     8014

/* E_SCM_INFOBASE_WRITE: Error writing infobase. */
#define E_SCM_INFOBASE_WRITE    8015

/* E_SCM_INV_INSTANCE: invalid instance specified */
#define E_SCM_INV_INSTANCE      8016

/* E_SCM_INV_VERINFO: invalid version info structure */
#define E_SCM_INV_VERINFO       8017

/* E_SCM_ITEM_NOTFOUND: item not found. */
#define E_SCM_ITEM_NOTFOUND     8018

/* E_SCM_REQ_NOTACCEPTED: Library is not accepting API requests */
#define E_SCM_REQ_NOTACCEPTED   8019

/* E_SCM_MOD_LOAD: Error loading module (e.g. LoadLibrary() error) */
#define E_SCM_MOD_LOAD          8020

/* E_SCM_MOD_INUSE: module is in use. */
#define E_SCM_MOD_INUSE         8021

/* E_SCM_NOTINITED: Component is not inited. */
#define E_SCM_NOTINITED         8022

/* E_SCM_INV_FNAME: no file name/invalid file name. */
#define E_SCM_INV_FNAME         8023

/* E_SCM_INV_FCNNAME: error importing symbols, invalid function name */
#define E_SCM_INV_FCNNAME       8024

/* E_SCM_INST_NOFREE: no free (unused) instances on free list. */
#define E_SCM_INST_NOFREE       8025

/* E_SCM_MOD_EXIST: module exists in directory */
#define E_SCM_MOD_EXIST         8026 

/* E_SCM_ATTR_MISSING: attribute missing. */
#define E_SCM_ATTR_MISSING      8027

/* E_SCM_BUFFERS_MISSING: Required input or output buffers are missing. */
#define E_SCM_BUFFERS_MISSING   8028

/* E_SCM_MOD_NOTCLAIMED: module not claimed. */
#define E_SCM_MOD_NOTCLAIMED    8029

/* E_SCM_BUFSIZE: return buffer too small */
#define E_SCM_BUFSIZE           8030

/* E_SCM_INTERNAL: internal error.  Call Pervasive. */
#define E_SCM_INTERNAL          8031

/*
**  E_SCM_INV_CID: a component ID string was found but may be invalid
**  because it is too short, too long or contains funny characters.
*/
#define E_SCM_INV_CID           8032

/*
**  E_SCM_SYM_IMPORT: Error importing symbol, most likely symbol not
**  exported.
*/
#define E_SCM_SYM_IMPORT        8033

/* E_SCM_INV_CALLTYPE: unsupported/invalid calltype */
#define E_SCM_INV_CALLTYPE      8034

/* E_SCM_INV_API: Unsupported, invalid or un#defined API */
#define E_SCM_INV_API           8035

/*
**  E_SCM_INV_VER: major/minor functional level errors.  These include:
**  a) lwr major is greater than upper major, b) lwr minor is greater
**  than upper minor, and c) version is out of range.
*/
#define E_SCM_INV_VER           8036

/* E_SCM_W16W32_GMHKERNEL: W16 cannot get module handle for W32 Kernel. */
#define E_SCM_W16W32_GMHKERNEL  8037

/*
**  E_SCM_WCCOMP: Error compiling wildcard expression for directory
**  scanning.  This error should not occur.
*/
#define E_SCM_WCCOMP            8038

/* E_SCM_INV_HANDLE: Invalid OS Handle. */
#define E_SCM_INV_HANDLE        8039

/* E_SCM_CID_CRC: CRC in CID is not correct. */
#define E_SCM_CID_CRC           8040

/* E_SCM_INV_PATH_LENGTH: Invalid Path length. */
#define E_SCM_INV_PATH_LENGTH   8041


/*
**  EventLog
*/

#define E_EVENTLOG_NO_ERROR                   E_NONE
#define E_EVENTLOG_INVALID_INSTANCE           E_SCM_INV_INSTANCE
#define E_EVENTLOG_NO_MEMORY                  E_SCM_MEM
#define E_EVENTLOG_INVALID_PATH_LEN           E_SCM_INV_PATH_LENGTH

#define E_EVENTLOG_INVALID_TYPE               8200
#define E_EVENTLOG_INVALID_COMPONENT          8201
#define E_EVENTLOG_INVALID_COMPONENT_NAME     8202
#define E_EVENTLOG_INVALID_MESSAGE_FILE       8203
#define E_EVENTLOG_INVALID_CATEGORY_FILE      8204
#define E_EVENTLOG_INVALID_HANDLE             8205
#define E_EVENTLOG_COMPONENT_MISMATCH         8206
#define E_EVENTLOG_COMPONENT_NOT_FOUND        8207
#define E_EVENTLOG_COMPONENT_NAME_EXISTS      8208
#define E_EVENTLOG_MESSAGE_FILE_NOT_FOUND     8209
#define E_EVENTLOG_CATEGORY_FILE_NOT_FOUND    8210
#define E_EVENTLOG_MISSING_MESSAGE_FILE       8211
#define E_EVENTLOG_MISSING_CATEGORY_FILE      8212
#define E_EVENTLOG_BINARY_DATA_TOO_LONG       8213
#define E_EVENTLOG_MESSAGE_TOO_LONG           8214
#define E_EVENTLOG_MESSAGE_NOT_FOUND          8215
#define E_EVENTLOG_CATEGORY_NOT_FOUND         8216
#define E_EVENTLOG_PARAMETER_MISMATCH         8217
#define E_EVENTLOG_WRITE_ERROR                8218

/*
**        ECASAPI Error codes  (Keep in sync with definitions in ecasapi.h)
*/
#define  ECASAPI_ERROR_BASE                               8500

#define  ECASAPI_SUCCESS                                  0
#define  ECASAPI_ERROR_INITIALIZING_SCM                   ECASAPI_ERROR_BASE + 0
#define  ECASAPI_ERROR_LOCATING_UPI_DLL                   ECASAPI_ERROR_BASE + 2
#define  ECASAPI_ERROR_ADDRESS_NOT_FOUND_IN_UPI_DLL       ECASAPI_ERROR_BASE + 3
#define  ECASAPI_ERROR_CREATING_SEMAPHORE                 ECASAPI_ERROR_BASE + 4
#define  ECASAPI_ERROR_OPENING_DBU_SESSION                ECASAPI_ERROR_BASE + 5
#define  ECASAPI_ERROR_LOADING_BTRIEVE_MODULE             ECASAPI_ERROR_BASE + 6
#define  ECASAPI_ERROR_NO_DBU_SESSION                     ECASAPI_ERROR_BASE + 7
#define  ECASAPI_ERROR_ENABLING_BTRIEVE_ACCESS            ECASAPI_ERROR_BASE + 8
#define  ECASAPI_ERROR_BTRIEVE_INIT_TIMEOUT               ECASAPI_ERROR_BASE + 9
#define  ECASAPI_ERROR_LOADING_SSQL_MODULE                ECASAPI_ERROR_BASE + 10
#define  ECASAPI_ERROR_ENABLING_SQL_ACCESS                ECASAPI_ERROR_BASE + 11
#define  ECASAPI_ERROR_SSQL_INIT_TIMEOUT                  ECASAPI_ERROR_BASE + 12
#define  ECASAPI_ERROR_DISABLING_BTRIEVE_ACCESS           ECASAPI_ERROR_BASE + 13
#define  ECASAPI_ERROR_UNLOADING_MKDE_MODULE              ECASAPI_ERROR_BASE + 14
#define  ECASAPI_ERROR_DISABLING_SQL_ACCESS               ECASAPI_ERROR_BASE + 15
#define  ECASAPI_ERROR_UNLOADING_SSQL_MODULE              ECASAPI_ERROR_BASE + 16
#define  ECASAPI_ERROR_CLOSING_DBU_SESSION                ECASAPI_ERROR_BASE + 17
#define  ECASAPI_ERROR_OUT_OF_MEMORY                      ECASAPI_ERROR_BASE + 18

#define  ECASAPI_ERRORS_MAX                               19

/*
**        DBSMGR Error codes
*/
#define  W3DBSMGR_ERROR_BASE                              8590

#define  W3DBSMGR_SUCCESS                                 0
#define  W3DBSMGR_ERROR_INITIALIZING_SCM                  W3DBSMGR_ERROR_BASE + 0
#define  W3DBSMGR_ERROR_ALREADY_LOADED                    W3DBSMGR_ERROR_BASE + 1
#define  W3DBSMGR_ERROR_OUT_OF_MEMORY                     W3DBSMGR_ERROR_BASE + 2
#define  W3DBSMGR_ERROR_CREATING_THREAD                   W3DBSMGR_ERROR_BASE + 3

#define  W3DBSMGR_ERRORS_MAX                              4


#endif /* #if !#defined(_GENSTAT_H_INCLUDED) */
