{*************************************************************************
**
**  Copyright 1982-1997 Pervasive Software Inc. All Rights Reserved
**
*************************************************************************}
{***********************************************************************
   SQLAPIP.PAS
      This is the Scalable SQL Pascal unit for DOS Protected Mode to be
      called by Borland Pascal for DOS or Borland Turbo Pascal for DOS.

   Additional information for using Borland 16-Bit
   protected mode is provided below.

   1. SYSTEM REQUIREMENTS
   ----------------------
   You must have Borland Pascal with Objects, Version 7.0 installed
   on your system for application development and you will need access
   to Borland's protected mode components (RTM.EXE and DPMI16BI.OVL)
   for application execution.

   For more detailed information on DOS protected mode programming,
   refer to Chapter 17, Programming in DOS protected mode, in Borland's
   Pascal with Objects, Version 7.0, Language Guide.

   2. NEEDED MODULES
   -----------------
   SQL16B.DLL   -  Scalable SQL DOS protected mode interface
                   16-bit dynamic link library.

   SQLAPIP.PAS  -  Scalable SQL DOS protected mode function prototypes.
                   You will need to compile this into a .tpp file.

   SQLSAMPP.PAS -  Scalable SQL DOS protected mode sample Pascal
                   program.

   SQLSAMPP.MAK -  Scalable SQL DOS protected mode sample Pascal
                   program makefile.

   Copy these files from the installation diskette to your development
   directory.

   The SQL sample files (SQLSAMPP.PAS and SQLSAMPP.MAK) provide
   an example of how to create a DOS protected mode application.

   3. APPLICATION DEVELOPMENT
   --------------------------
   If you are developing a Scalable SQL DOS protected mode application,
   the following steps may help you.

   a. Make sure that the directory pathanme to Borland's protected mode
      components (RTM.EXE and DPMI16BI.OVL) and /bin directory are
      on your path.

   b. Make sure that your Pascal application references the prototype
      unit by including the file name in you USES section as follows

       USES SQLAPIP;

   c. Make sure that the SQL16B.DLL and SQLAPIP.TPP files are in your
      current working directory or their directory pathnames are on
      your path.

  d. Compile your application as protected mode by specifying the /cp
     compiler switch as follows

       bcp /cp <application.PAS>

   If you are developing a Scalable SQL DOS protected mode application
   from a real mode application you must first delete any reference to
   the SQLAPID unit in your application USES section.  Follow steps
   'a' through 'd', above, to complete the effort.
************************************************************************}

UNIT sqlapip;

{***********************************************************************
   PASCAL INTERFACE SECTION
************************************************************************}
INTERFACE
{***********************************************************************
   SESSION MANAGEMENT PRIMITIVES
      The following primitives are required by applications which
      want to use multiple SQL logins:
         xGetSessionID
         xPutSessionID
************************************************************************}
   FUNCTION xGetSessionID(
               VAR session : INTEGER ) : INTEGER;

   FUNCTION xPutSessionID(
               session : INTEGER ) : INTEGER;

{***********************************************************************
   MISCELLANEOUS FUNCTIONS
      SQLGetCountDatabaseNames
      SQLGetDatabaseNames
      SQLUnloadDBNames
************************************************************************}
FUNCTION SQLGetCountDatabaseNames(
           VAR iCount : INTEGER ) : INTEGER;

FUNCTION SQLGetDatabaseNames(
           VAR iBufLen  : INTEGER;
           VAR bDataBuf : CHAR ) : INTEGER;

FUNCTION SQLUnloadDBNAMES(
           iReserved : INTEGER ) : INTEGER;

{***********************************************************************
   SQL-LEVEL FUNCTIONS
      XQLCursor     XQLFetch    XQLSPUtility   XQLVersion
      XQLCompile    XQLFormat   XQLStatus      XQLConvert
      XQLDescribe   XQLFree     XQLStop        XQLValidate
      XQLExec       XQLLogin    XQLSubst       XQLMask
                    XQLLogout
***********************************************************************}
   FUNCTION XQLCursor(
               VAR CURSOR : INTEGER ) : INTEGER;

   FUNCTION XQLCompile(
               CURSOR    : INTEGER;
           VAR LEN       : INTEGER;
           VAR STMNT )   : INTEGER;

   FUNCTION XQLDescribe(
               CURSOR  : INTEGER;
               POS     : INTEGER;
           VAR DTYPE   : INTEGER;
           VAR SIZE    : INTEGER;
           VAR DEC     : INTEGER;
           VAR DSIZE   : INTEGER;
           VAR NLENGTH : INTEGER;
           VAR NAME    : CHAR ) : INTEGER;

    FUNCTION XQLExec(
                CURSOR : INTEGER ) : INTEGER;

   FUNCTION XQLFetch(
               CURSOR : INTEGER;
               OP     : INTEGER;
           VAR DLEN   : INTEGER;
           VAR BUF;
           VAR SEL    : LONGINT;
               ASCII  : INTEGER;
               SPACES : INTEGER ) : INTEGER;

   FUNCTION XQLFormat(
               CURSOR : INTEGER;
               POS    : INTEGER;
               LENGTH : INTEGER;
           VAR MASKBUF ) : INTEGER;

   FUNCTION XQLFree(
               CURSOR : INTEGER ) : INTEGER;

   FUNCTION XQLLogin(
               VAR USERID;
               VAR PSWDNAME;
               VAR DD;
               VAR DATAPATH;
               VAR SRES;
                   iFeaturesUsed : INTEGER ) : INTEGER;

   FUNCTION XQLLogout : INTEGER;

   FUNCTION XQLSPUtility(
               CURSOR    : INTEGER;
               OPTION    : INTEGER;
           VAR STMNTCNT  : INTEGER;
           VAR STMNTEXEC : INTEGER;
           VAR BUFLEN    : INTEGER;
           VAR STATBUF ) : INTEGER;

   FUNCTION XQLStatus(
               CURSOR : INTEGER;
               OPTION : INTEGER;
           VAR STATBUF ) : INTEGER;

   FUNCTION XQLStop : INTEGER;

   FUNCTION XQLSubst(
               CURSOR  : INTEGER;
               SCOUNT  : INTEGER;
               NAMELEN : INTEGER;
           VAR VARNAME;
               TEXTLEN : INTEGER;
           VAR VARTEXT ) : INTEGER;

   FUNCTION XQLVersion(
               VAR VBUFFER ) : INTEGER;

   FUNCTION XQLConvert(
               OPTION : INTEGER;
               DTYPE  : INTEGER;
               SIZE   : INTEGER;
               DEC    : INTEGER;
               DSIZE  : INTEGER;
           VAR VALUE;
           VAR RETVALUE;
           VAR MASK;
               JUST : INTEGER ) : INTEGER;

   FUNCTION XQLValidate(
               VAR COUNT : INTEGER;
               VAR FIELDNAME;
                   BUFLEN : INTEGER;
               VAR DATA ) : INTEGER;

   FUNCTION XQLMask(
               OPTION : INTEGER;
               DTYPE  : INTEGER;
               SIZE   : INTEGER;
               DEC    : INTEGER;
           VAR MLEN   : INTEGER;
           VAR MASK ) : INTEGER;

{***********************************************************************
   HISTORICAL RELATIONAL PRIMITIVES
      The following functions will be phased out over time.  They
      are included here to support existing applications.  New applications
      should not use these functions.

      xCompute    xOrder         xDD            xAccess
      xDescribe   xRemall        xDDAttr        xPassword
      xMovefld    xRemove        xDDModify      xSecurity
      xFetch      xReset         xDDCreate      xUser
      xField      xRestrict      xDDDrop        xChar
      xFree       xStop          xDDField       xVersion
      xInsert     xTrans         xDDFile        xStatus
      xJoin       xStore         xDDIndex       xConvert
      xLogin      xRecall        xDDPath        xValidate
      xLogout     xUpdate        xDDView        xMask
      xNew        xUpdall
***********************************************************************}
   FUNCTION xCompute(
               VHANDLE   : INTEGER;
           VAR FNAME;
               DATA_TYPE : INTEGER;
               DATA_SIZE : INTEGER;
               DEC       : INTEGER;
           VAR ELEN      : INTEGER;
           VAR EBUF )    : INTEGER;

   FUNCTION xDescribe(
               VHANDLE : INTEGER;
               SUBF    : INTEGER;
           VAR DLEN    : INTEGER;
               POS     : INTEGER;
           VAR CNT     : INTEGER;
           VAR BUF )   : INTEGER;

   FUNCTION xMovefld(
               VHANDLE : INTEGER;
               FPOS    : INTEGER;
               TPOS    : INTEGER ) : INTEGER;

   FUNCTION xFetch(
               VHANDLE : INTEGER;
           VAR DLEN    : INTEGER;
               FETCH_OP: INTEGER;
           VAR SELCNT  : LONGINT;
           VAR REJCNT  : LONGINT;
           VAR BUF )   : INTEGER;

   FUNCTION xField(
               VHANDLE : INTEGER;
               SUBF    : INTEGER;
               POS     : INTEGER;
           VAR CNT     : INTEGER;
           VAR BUF )   : INTEGER;

   FUNCTION xFree(
               VHANDLE : INTEGER) : INTEGER;

   FUNCTION xInsert(
               VHANDLE  : INTEGER;
           VAR COUNT    : INTEGER;
           VAR FILES;
           VAR EDITCNT  : LONGINT;
           VAR BUFFER ) : INTEGER;

   FUNCTION xJoin(
               VHANDLE  : INTEGER;
           VAR NEWFILE;
           VAR OWNERNAME;
               JTYPE    : INTEGER;
               VCOUNT   : INTEGER;
           VAR VNAMES;
               NCOUNT   : INTEGER;
           VAR NNAMES ) : INTEGER;

   FUNCTION xLogin(
           VAR USERID;
           VAR PASSWORD;
           VAR DPATH;
           VAR FPATH;
           VAR SRES;
               iFeaturesUsed : INTEGER ) : INTEGER;

   FUNCTION xLogout : INTEGER;

   FUNCTION xNew(
           VAR VHANDLE : INTEGER;
           VAR FNAME;
           VAR OWNERNAME;
               MODE : INTEGER ) : INTEGER;

   FUNCTION xOrder(
               VHANDLE : INTEGER;
               CNT     : INTEGER;
           VAR BUF )   : INTEGER;

   FUNCTION xRemall(
               VHANDLE : INTEGER;
           VAR COUNT   : INTEGER;
           VAR FILES;
               ROP     : INTEGER;
           VAR SELCNT  : LONGINT;
               REJCNT  : LONGINT ) : INTEGER;

   FUNCTION xRemove(
               VHANDLE : INTEGER;
           VAR COUNT   : INTEGER;
           VAR FILES;
           VAR EDITCNT : LONGINT ) : INTEGER;

   FUNCTION xReset(
           VAR SRESERVED ) : INTEGER;

   FUNCTION xRestrict(
               VHANDLE : INTEGER;
               NEW_EXP : INTEGER;
           VAR ELEN    : INTEGER;
           VAR EBUF )  : INTEGER;

   FUNCTION xStop : INTEGER;

   FUNCTION xTrans(
               OPERATION : INTEGER ) : INTEGER;

   FUNCTION xStore(
               VHANDLE : INTEGER;
           VAR VNAME;
               BLEN    : INTEGER;
           VAR VBUF )  : INTEGER;

   FUNCTION xRecall(
           VAR VHANDLE : INTEGER;
           VAR VNAME;
               ONUM    : INTEGER;
           VAR OLST;
               MODE    : INTEGER;
           VAR BLEN    : INTEGER;
           VAR VBUF )  : INTEGER;

   FUNCTION xUpdate(
               VHANDLE  : INTEGER;
           VAR COUNT    : INTEGER;
           VAR FILES;
           VAR EDITCNT  : LONGINT;
           VAR BUFFER ) : INTEGER;

   FUNCTION xUpdall(
               VHANDLE   : INTEGER;
           VAR UPDCOUNT  : INTEGER;
           VAR FILES;
               UOP       : INTEGER;
           VAR SELCNT    : LONGINT;
           VAR REJCNT    : LONGINT;
               COLCNT    : INTEGER;
           VAR RESULT;
           VAR COMPARE ) : INTEGER;

   FUNCTION xDD(
           VAR DIRPATH;
           FNCTION : INTEGER ) : INTEGER;

   FUNCTION xDDAttr(
               OPTION    : INTEGER;
            VAR FLDNAME;
                ATTRIB   : INTEGER;
            VAR BSIZE    : INTEGER;
            VAR ABUF )   : INTEGER;

   FUNCTION xDDModify(
               OPTION      : INTEGER;
           VAR FNAME;
               CREATE_FILE : INTEGER;
           VAR PATH;
           VAR OWNERNAME;
               OACC        : INTEGER;
               FLDCOUNT    : INTEGER;
           VAR FLDLIST;
               KEYCOUNT    : INTEGER;
           VAR KLIST )     : INTEGER;

   FUNCTION xDDCreate(
               OPTION      : INTEGER;
           VAR FNAME;
               CREATE_FILE : INTEGER;
           VAR PATH;
           VAR OWNERNAME;
               OACC        : INTEGER;
               FLDCOUNT    : INTEGER;
           VAR FLDLIST;
               KEYCOUNT    : INTEGER;
           VAR KLIST;
               BLEN        : INTEGER;
           VAR CRPARMS )   : INTEGER;

   FUNCTION xDDDrop(
           VAR FILENAME;
               FILETYPE   : INTEGER;
               DELETEPATH : INTEGER ) : INTEGER;

    FUNCTION xDDField(
                OPTION : INTEGER;
            VAR FCOUNT : INTEGER;
            VAR FBUF;
            VAR BLEN   : INTEGER;
            VAR OBUF ) : INTEGER;

   FUNCTION xDDFile(
               OPTION : INTEGER;
           VAR FCOUNT : INTEGER;
           VAR FBUF;
           VAR BLEN   : INTEGER;
           VAR OBUF ) : INTEGER;

   FUNCTION xDDIndex(
               OPTION : INTEGER;
           VAR FCOUNT : INTEGER;
           VAR FBUF;
           VAR BLEN   : INTEGER;
           VAR OBUF ) : INTEGER;

   FUNCTION xDDPath(
               DIRTYPE    : INTEGER;
           VAR PATHNAME ) : INTEGER;

   FUNCTION xDDView(
           VAR FCOUNT : INTEGER;
           VAR VNAME;
           VAR BLEN   : INTEGER;
           VAR OBUF ) : INTEGER;

   FUNCTION xAccess(
           VAR MASTERPSWD;
           VAR UNAME;
               OPTION      : INTEGER;
               ARIGHTS     : INTEGER;
           VAR FNAME;
           VAR FCOUNT      : INTEGER;
           VAR FIELDNAME;
           VAR BLEN        : INTEGER;
           VAR OBUF )      : INTEGER;

   FUNCTION xPassword(
           VAR UNAME;
           VAR PASSWORD ) : INTEGER;

   FUNCTION xSecurity(
           VAR MPSWD;
           FLAG : INTEGER ) : INTEGER;

   FUNCTION xUser(
           VAR MASTERPSWD;
               OPTION      : INTEGER;
           VAR UNAME;
           VAR PASSWORD;
               UGRIGHTS    : INTEGER;
           VAR FCOUNT      : INTEGER;
           VAR BLEN        : INTEGER;
           VAR UBUF )      : INTEGER;

   FUNCTION xChar(
               CHARFCN  : INTEGER;
               CHARTYPE : INTEGER;
           VAR VALUE  ) : INTEGER;

   FUNCTION xVersion(
           VAR BUFFER ) : INTEGER;

   FUNCTION xStatus(
               VHANDLE   : INTEGER;
               SUBFUNC   : INTEGER;
           VAR LEN       : INTEGER;
           VAR STATBUF ) : INTEGER;

   FUNCTION xConvert(
               OPTION     : INTEGER;
               DTYPE      : INTEGER;
               SIZE       : INTEGER;
               DEC        : INTEGER;
               DSIZE      : INTEGER;
           VAR VALUE;
           VAR RETVALUE;
           VAR MASK;
               JUST       : INTEGER ) : INTEGER;

   FUNCTION xValidate(
           VAR COUNT       : INTEGER;
           VAR FIELDNAME;
               BUFLEN      : INTEGER;
           VAR DATA )      : INTEGER;

   FUNCTION xMask(
               OPTION : INTEGER;
               DTYPE  : INTEGER;
               SIZE   : INTEGER;
               DEC    : INTEGER;
           VAR MLEN   : INTEGER;
           VAR MASK ) : INTEGER;

{***********************************************************************
   PASCAL IMPLEMENTATION SECTION
************************************************************************}
IMPLEMENTATION
{***********************************************************************
   SESSION MANAGEMENT PRIMITIVES
************************************************************************}
   FUNCTION xGetSessionID(
               VAR session : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XGETSESSIONID';

   FUNCTION xPutSessionID(
               session : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XPUTSESSIONID';

{***********************************************************************
   MISCELLANEOUS FUNCTIONS
************************************************************************}
FUNCTION SQLGetCountDatabaseNames(
           VAR iCount : INTEGER ) : INTEGER;
           external 'SQL16B' name 'SQLGETCOUNTDATABASENAMES';

FUNCTION SQLGetDatabaseNames(
           VAR iBufLen  : INTEGER;
           VAR bDataBuf : CHAR ) : INTEGER;
           external 'SQL16B' name 'SQLGETDATABASENAMES';

FUNCTION SQLUnloadDBNAMES(
           iReserved : INTEGER ) : INTEGER;
           external 'SQL16B' name 'SQLUNLOADDBNAMES';

{************************************************************************
    SQL-Level Functions Implementation
 ************************************************************************}
   FUNCTION XQLCursor(
               VAR CURSOR : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XQLCURSOR';

   FUNCTION XQLCompile(
               CURSOR    : INTEGER;
           VAR LEN       : INTEGER;
           VAR STMNT )   : INTEGER;
               external 'SQL16B' name 'XQLCOMPILE';

   FUNCTION XQLDescribe(
               CURSOR  : INTEGER;
               POS     : INTEGER;
           VAR DTYPE   : INTEGER;
           VAR SIZE    : INTEGER;
           VAR DEC     : INTEGER;
           VAR DSIZE   : INTEGER;
           VAR NLENGTH : INTEGER;
           VAR NAME    : CHAR ) : INTEGER;
               external 'SQL16B' name 'XQLDESCRIBE';

    FUNCTION XQLExec(
                CURSOR : INTEGER ) : INTEGER;
                external 'SQL16B' name 'XQLEXEC';

   FUNCTION XQLFetch(
               CURSOR : INTEGER;
               OP     : INTEGER;
           VAR DLEN   : INTEGER;
           VAR BUF;
           VAR SEL    : LONGINT;
               ASCII  : INTEGER;
               SPACES : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XQLFETCH';

   FUNCTION XQLFormat(
               CURSOR : INTEGER;
               POS    : INTEGER;
               LENGTH : INTEGER;
           VAR MASKBUF ) : INTEGER;
               external 'SQL16B' name 'XQLFORMAT';

   FUNCTION XQLFree(
               CURSOR : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XQLFREE';

   FUNCTION XQLLogin(
               VAR USERID;
               VAR PSWDNAME;
               VAR DD;
               VAR DATAPATH;
               VAR SRES;
                   iFeaturesUsed : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XQLLOGIN';

   FUNCTION XQLLogout : INTEGER;
               external 'SQL16B' name 'XQLLOGOUT';

   FUNCTION XQLSPUtility(
               CURSOR    : INTEGER;
               OPTION    : INTEGER;
           VAR STMNTCNT  : INTEGER;
           VAR STMNTEXEC : INTEGER;
           VAR BUFLEN    : INTEGER;
           VAR STATBUF ) : INTEGER;
               external 'SQL16B' name 'XQLSPUTILITY';

   FUNCTION XQLStatus(
               CURSOR : INTEGER;
               OPTION : INTEGER;
           VAR STATBUF ) : INTEGER;
               external 'SQL16B' name 'XQLSTATUS';

   FUNCTION XQLStop : INTEGER;
               external 'SQL16B' name 'XQLSTOP';

   FUNCTION XQLSubst(
               CURSOR  : INTEGER;
               SCOUNT  : INTEGER;
               NAMELEN : INTEGER;
           VAR VARNAME;
               TEXTLEN : INTEGER;
           VAR VARTEXT ) : INTEGER;
               external 'SQL16B' name 'XQLSUBST';

   FUNCTION XQLVersion(
               VAR VBUFFER ) : INTEGER;
               external 'SQL16B' name 'XQLVERSION';

   FUNCTION XQLConvert(
               OPTION : INTEGER;
               DTYPE  : INTEGER;
               SIZE   : INTEGER;
               DEC    : INTEGER;
               DSIZE  : INTEGER;
           VAR VALUE;
           VAR RETVALUE;
           VAR MASK;
               JUST : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XQLCONVERT';

   FUNCTION XQLValidate(
               VAR COUNT : INTEGER;
               VAR FIELDNAME;
                   BUFLEN : INTEGER;
               VAR DATA ) : INTEGER;
               external 'SQL16B' name 'XQLVALIDATE';

   FUNCTION XQLMask(
               OPTION : INTEGER;
               DTYPE  : INTEGER;
               SIZE   : INTEGER;
               DEC    : INTEGER;
           VAR MLEN   : INTEGER;
           VAR MASK ) : INTEGER;
               external 'SQL16B' name 'XQLMASK';

{************************************************************************
   Primitive-Level Functions Implementation
 ************************************************************************}
   FUNCTION xCompute(
               VHANDLE   : INTEGER;
           VAR FNAME;
               DATA_TYPE : INTEGER;
               DATA_SIZE : INTEGER;
               DEC       : INTEGER;
           VAR ELEN      : INTEGER;
           VAR EBUF )    : INTEGER;
               external 'SQL16B' name 'XCOMPUTE';

   FUNCTION xDescribe(
               VHANDLE : INTEGER;
               SUBF    : INTEGER;
           VAR DLEN    : INTEGER;
               POS     : INTEGER;
           VAR CNT     : INTEGER;
           VAR BUF )   : INTEGER;
               external 'SQL16B' name 'XDESCRIBE';

   FUNCTION xMovefld(
               VHANDLE : INTEGER;
               FPOS    : INTEGER;
               TPOS    : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XMOVEFLD';

   FUNCTION xFetch(
               VHANDLE : INTEGER;
           VAR DLEN    : INTEGER;
               FETCH_OP: INTEGER;
           VAR SELCNT  : LONGINT;
           VAR REJCNT  : LONGINT;
           VAR BUF )   : INTEGER;
               external 'SQL16B' name 'XFETCH';

   FUNCTION xField(
               VHANDLE : INTEGER;
               SUBF    : INTEGER;
               POS     : INTEGER;
           VAR CNT     : INTEGER;
           VAR BUF )   : INTEGER;
               external 'SQL16B' name 'XFIELD';

   FUNCTION xFree(
               VHANDLE : INTEGER) : INTEGER;
               external 'SQL16B' name 'XFREE';

   FUNCTION xInsert(
               VHANDLE  : INTEGER;
           VAR COUNT    : INTEGER;
           VAR FILES;
           VAR EDITCNT  : LONGINT;
           VAR BUFFER ) : INTEGER;
               external 'SQL16B' name 'XINSERT';

   FUNCTION xJoin(
               VHANDLE  : INTEGER;
           VAR NEWFILE;
           VAR OWNERNAME;
               JTYPE    : INTEGER;
               VCOUNT   : INTEGER;
           VAR VNAMES;
               NCOUNT   : INTEGER;
           VAR NNAMES ) : INTEGER;
               external 'SQL16B' name 'XJOIN';

   FUNCTION xLogin(
           VAR USERID;
           VAR PASSWORD;
           VAR DPATH;
           VAR FPATH;
           VAR SRES;
               iFeaturesUsed : INTEGER ) : INTEGER;
            external 'SQL16B' name 'XLOGIN';

   FUNCTION xLogout : INTEGER;
            external 'SQL16B' name 'XLOGOUT';

   FUNCTION xNew(
           VAR VHANDLE : INTEGER;
           VAR FNAME;
           VAR OWNERNAME;
               MODE : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XNEW';

   FUNCTION xOrder(
               VHANDLE : INTEGER;
               CNT     : INTEGER;
           VAR BUF )   : INTEGER;
               external 'SQL16B' name 'XORDER';

   FUNCTION xRemall(
               VHANDLE : INTEGER;
           VAR COUNT   : INTEGER;
           VAR FILES;
               ROP     : INTEGER;
           VAR SELCNT  : LONGINT;
               REJCNT  : LONGINT ) : INTEGER;
               external 'SQL16B' name 'XREMALL';

   FUNCTION xRemove(
               VHANDLE : INTEGER;
           VAR COUNT   : INTEGER;
           VAR FILES;
           VAR EDITCNT : LONGINT ) : INTEGER;
               external 'SQL16B' name 'XREMOVE';

   FUNCTION xReset(
           VAR SRESERVED ) : INTEGER;
               external 'SQL16B' name 'XRESET';

   FUNCTION xRestrict(
               VHANDLE : INTEGER;
               NEW_EXP : INTEGER;
           VAR ELEN    : INTEGER;
           VAR EBUF )  : INTEGER;
               external 'SQL16B' name 'XRESTRICT';

   FUNCTION xStop : INTEGER;
               external 'SQL16B' name 'XSTOP';

   FUNCTION xTrans(
               OPERATION : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XTRANS';

   FUNCTION xStore(
               VHANDLE : INTEGER;
           VAR VNAME;
               BLEN    : INTEGER;
           VAR VBUF )  : INTEGER;
               external 'SQL16B' name 'XSTORE';

   FUNCTION xRecall(
           VAR VHANDLE : INTEGER;
           VAR VNAME;
               ONUM    : INTEGER;
           VAR OLST;
               MODE    : INTEGER;
           VAR BLEN    : INTEGER;
           VAR VBUF )  : INTEGER;
               external 'SQL16B' name 'XRECALL';

   FUNCTION xUpdate(
               VHANDLE  : INTEGER;
           VAR COUNT    : INTEGER;
           VAR FILES;
           VAR EDITCNT  : LONGINT;
           VAR BUFFER ) : INTEGER;
               external 'SQL16B' name 'XUPDATE';

   FUNCTION xUpdall(
               VHANDLE   : INTEGER;
           VAR UPDCOUNT  : INTEGER;
           VAR FILES;
               UOP       : INTEGER;
           VAR SELCNT    : LONGINT;
           VAR REJCNT    : LONGINT;
               COLCNT    : INTEGER;
           VAR RESULT;
           VAR COMPARE ) : INTEGER;
               external 'SQL16B' name 'XUPDALL';

   FUNCTION xDD(
           VAR DIRPATH;
           FNCTION : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XDD';

   FUNCTION xDDAttr(
               OPTION    : INTEGER;
            VAR FLDNAME;
                ATTRIB   : INTEGER;
            VAR BSIZE    : INTEGER;
            VAR ABUF )   : INTEGER;
               external 'SQL16B' name 'XDDATTR';

   FUNCTION xDDModify(
               OPTION      : INTEGER;
           VAR FNAME;
               CREATE_FILE : INTEGER;
           VAR PATH;
           VAR OWNERNAME;
               OACC        : INTEGER;
               FLDCOUNT    : INTEGER;
           VAR FLDLIST;
               KEYCOUNT    : INTEGER;
           VAR KLIST )     : INTEGER;
               external 'SQL16B' name 'XDDMODIFY';

   FUNCTION xDDCreate(
               OPTION      : INTEGER;
           VAR FNAME;
               CREATE_FILE : INTEGER;
           VAR PATH;
           VAR OWNERNAME;
               OACC        : INTEGER;
               FLDCOUNT    : INTEGER;
           VAR FLDLIST;
               KEYCOUNT    : INTEGER;
           VAR KLIST;
               BLEN        : INTEGER;
           VAR CRPARMS )   : INTEGER;
               external 'SQL16B' name 'XDDCREATE';

   FUNCTION xDDDrop(
           VAR FILENAME;
               FILETYPE   : INTEGER;
               DELETEPATH : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XDDDROP';

    FUNCTION xDDField(
                OPTION : INTEGER;
            VAR FCOUNT : INTEGER;
            VAR FBUF;
            VAR BLEN   : INTEGER;
            VAR OBUF ) : INTEGER;
                external 'SQL16B' name 'XDDFIELD';

   FUNCTION xDDFile(
               OPTION : INTEGER;
           VAR FCOUNT : INTEGER;
           VAR FBUF;
           VAR BLEN   : INTEGER;
           VAR OBUF ) : INTEGER;
               external 'SQL16B' name 'XDDFILE';

   FUNCTION xDDIndex(
               OPTION : INTEGER;
           VAR FCOUNT : INTEGER;
           VAR FBUF;
           VAR BLEN   : INTEGER;
           VAR OBUF ) : INTEGER;
               external 'SQL16B' name 'XDDINDEX';

   FUNCTION xDDPath(
               DIRTYPE    : INTEGER;
           VAR PATHNAME ) : INTEGER;
               external 'SQL16B' name 'XDDPATH';

   FUNCTION xDDView(
           VAR FCOUNT : INTEGER;
           VAR VNAME;
           VAR BLEN   : INTEGER;
           VAR OBUF ) : INTEGER;
               external 'SQL16B' name 'XDDVIEW';

   FUNCTION xAccess(
           VAR MASTERPSWD;
           VAR UNAME;
               OPTION      : INTEGER;
               ARIGHTS     : INTEGER;
           VAR FNAME;
           VAR FCOUNT      : INTEGER;
           VAR FIELDNAME;
           VAR BLEN        : INTEGER;
           VAR OBUF )      : INTEGER;
               external 'SQL16B' name 'XACCESS';

   FUNCTION xPassword(
           VAR UNAME;
           VAR PASSWORD ) : INTEGER;
               external 'SQL16B' name 'XPASSWORD';

   FUNCTION xSecurity(
           VAR MPSWD;
               FLAG : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XSECURITY';

   FUNCTION xUser(
           VAR MASTERPSWD;
               OPTION      : INTEGER;
           VAR UNAME;
           VAR PASSWORD;
               UGRIGHTS    : INTEGER;
           VAR FCOUNT      : INTEGER;
           VAR BLEN        : INTEGER;
           VAR UBUF )      : INTEGER;
               external 'SQL16B' name 'XUSER';

   FUNCTION xChar(
               CHARFCN  : INTEGER;
               CHARTYPE : INTEGER;
           VAR VALUE  ) : INTEGER;
               external 'SQL16B' name 'XCHAR';

   FUNCTION xVersion(
           VAR BUFFER ) : INTEGER;
               external 'SQL16B' name 'XVERSION';

   FUNCTION xStatus(
               VHANDLE   : INTEGER;
               SUBFUNC   : INTEGER;
           VAR LEN       : INTEGER;
           VAR STATBUF ) : INTEGER;
               external 'SQL16B' name 'XSTATUS';

   FUNCTION xConvert(
               OPTION     : INTEGER;
               DTYPE      : INTEGER;
               SIZE       : INTEGER;
               DEC        : INTEGER;
               DSIZE      : INTEGER;
           VAR VALUE;
           VAR RETVALUE;
           VAR MASK;
               JUST       : INTEGER ) : INTEGER;
               external 'SQL16B' name 'XCONVERT';

   FUNCTION xValidate(
           VAR COUNT       : INTEGER;
           VAR FIELDNAME;
               BUFLEN      : INTEGER;
           VAR DATA )      : INTEGER;
               external 'SQL16B' name 'XVALIDATE';

   FUNCTION xMask(
               OPTION : INTEGER;
               DTYPE  : INTEGER;
               SIZE   : INTEGER;
               DEC    : INTEGER;
           VAR MLEN   : INTEGER;
           VAR MASK ) : INTEGER;
               external 'SQL16B' name 'XMASK';

END.

