#ifndef _SQLCONST_H_INCLUDED
/*************************************************************************
**
**  Copyright 1982-1997 Pervasive Software Inc. All Rights Reserved
**
*************************************************************************/
/*************************************************************************
   SQLCONST.H - Scalable SQL Constants
      This file contains various Scalable SQL constants, mostly API return
      codes, for use by Scalable SQL 3.x C/C++ applications.

     Return Code Ranges:
      -100 - -199  ==>  Informative
         1 -  199  ==>  Reserved for Microkernel Engine ( in BTRCONST.H )
       200 -  999  ==>  Scalable SQL Engine
      1000 - 1999  ==>  Reserved for Microkernel Engine for OS2
      2000 - 2099  ==>  OS2/MS-Windows
      2100 - 2199  ==>  Scalable SQL Requester
      2200 - 2299  ==>  Miscellaneous Environment-Specific
      2300 - 2399  ==>  Named Database
      2900 - 2999  ==>  Inscribe
      3000 - 9999  ==>  Reserved for Scalable SQL and Components

*************************************************************************/
enum
{
    SS_SUCCESS = 0,
    SS_FAILURE = -1,

    /*
    **  Informative
    */
    SQL_INFORM_SET_COMPLETE = -101,
    SQL_INFORM_INSERT_COMPLETE = -102,
    SQL_INFORM_UPDATE_COMPLETE = -103,
    SQL_INFORM_DELETE_COMPLETE = -104,
    SQL_INFORM_CREATE_COMPLETE = -105,
    SQL_INFORM_ALTER_COMPLETE = -106,
    SQL_INFORM_DROP_COMPLETE = -107,
    SQL_INFORM_UNRESOLVED_SUBSTITUTION_VARS = -108,
    SQL_INFORM_NOMOREROWS = -109,
    SQL_INFORM_GRANT_COMPLETE = -110,
    SQL_INFORM_REVOKE_COMPLETE = -111,
    SQL_INFORM_STARTTRAN_COMPLETE = -112,
    SQL_INFORM_COMMIT_COMPLETE = -113,
    SQL_INFORM_ROLLBACK_COMPLETE = -114,
    SQL_INFORM_RECALLED_STORED_PROC = -115,
    SQL_INFORM_EXECUTED_STORED_PROC = -116,
    SQL_DECLARE_COMPLETE = -118,
    SQL_SET_ASSIGNMENT_COMPLETE = -119,
    SQL_OPEN_COMPLETE = -120,
    SQL_CLOSE_COMPLETE = -121,
    SQL_FETCH_COMPLETE = -122,
    SQL_POSITIONED_UPDATE_COMPLETE = -123,
    SQL_POSITIONED_DELETE_COMPLETE = -124,
    SQL_STORED_PROCEDURE_COMPLETE = -125,

    /*
    **  Microkernel Engine ( 1 - 199, in BTRCONST.H )
    */

    /*
    **  Scalable SQL Engine
    */
    SQL_PRIM_COMPONENT_NOT_LOADED = 200,
    SQL_INVALID_PRIMITIVE_FUNCTION= 201,
    SQL_INVALID_VIEW_HANDLE = 202,
    SQL_INVALID_MASTER_PASSWORD = 203,
    SQL_INVALID_DICTIONARY_FILENAME = 204,
    SQL_INVALID_PASWORD = 205,
    SQL_MAX_HANDLES_EXCEEDED = 206,
    SQL_FIELD_NOT_IN_DICTIONARY = 207,
    SQL_INVALID_POSITION_VALUE = 208,
    SQL_INVALID_USER_NAME = 209,
    SQL_BUFFER_TOO_SHORT = 210,
    SQL_MAX_RECORD_SIZE_EXCEEDED = 211,
    SQL_MAX_KEY_SIZE_EXCEEDED = 212,
    SQL_NO_DICTIONARY_READ_RIGHTS = 213,
    SQL_NO_FIELD_UPDATE_RIGHTS = 214,
    SQL_INVALID_LOCK_FLAG = 215,
    SQL_MAX_FIELDS_EXCEEDED = 216,
    SQL_CANNOT_JOIN_ORDERED_VIEW = 217,
    SQL_MAX_JOINS_EXCEEDED = 218,
    SQL_MAX_JOIN_SEGMENTS_EXCEEDED = 219,
    SQL_NO_MATCHING_INDEXES = 220,
    SQL_INVALID_SYNTAX = 221,
    SQL_NO_HEAP_MEM_P = 222,
    SQL_INVALID_DATA_TYPE = 223,
    SQL_INVALID_CHAR_IN_NUMERIC = 224,
    SQL_MAX_ORDER_BY_SEGMENTS_EXCEEDED = 225,
    SQL_INVALID_RECORD_COUNT_P = 226,
    SQL_INVALID_OPERAND = 227,
    SQL_INVALID_BUF_LENGTH = 228,
    SQL_FIELD_ALREADY_EXISTS = 229,
    SQL_INCOMPLETE_RESTRICTION = 230,
    SQL_NO_FIELDS_DEFINED_IN_VIEW = 231,
    SQL_FIELD_MUST_BE_IN_VIEW = 232,
    SQL_NOT_USED1 = 233,
    SQL_CANNOT_CREATE_EXTERNAL_INDEX_FILE = 234,
    SQL_INVALID_IO_OPERATION = 235,
    SQL_CANNOT_EDIT_VIEW = 236,
    SQL_INDEX_DESCRIPTOR_MISSING = 237,
    SQL_INDEX_DOES_NOT_EXIST = 238,
    SQL_TABLE_NOT_IN_DICTIONARY = 239,
    SQL_COLUMN_NAME_NOT_IN_DICTIONARY = 240,
    SQL_CANNOT_EDIT_DICTIONARY = 241,
    SQL_FIELD_NO_WRITE_ACCESS = 242,
    SQL_FIELD_NOT_MODIFIABLE = 243,
    SQL_TABLE_NO_WRITE_ACCESS = 244,
    SQL_CANNOT_DELETE_DICTINARY = 245,
    SQL_ACCESS_DENIED_P = 246,
    SQL_FIELD_ACCESS_DENIED = 247,
    SQL_INVALID_DATA_SIZE = 248,
    SQL_INCOMPATIBLE_FIELD_TYPES = 249,
    SQL_CANNOT_RECALL_VIEW = 250,
    SQL_ERROR_READING_VIEW = 251,
    SQL_CANNOT_FIND_VIEW = 252,
    SQL_INCORRECT_VIEW_DEF_VERSION = 253,
    SQL_CANNOT_CREATE_VIEW_DDF = 254,
    SQL_FIELD_IS_NOT_A_KEY = 255,
    SQL_INVALID_RELATION = 256,
    SQL_TABLE_ALREADY_DEFINED = 257,
    SQL_ERROR_UPDATING_DICTIONARY = 258,
    SQL_ERROR_INSERTING_NEW_TABLE = 259,
    SQL_ERROR_UPDATING_SYSTEM_RIGHTS = 260,
    SQL_VARLEN_FIELD_MUST_BE_LAST = 261,
    SQL_VARLEN_FIELD_CANT_BE_KEY = 262,
    SQL_VARLEN_FIELD_CANT_ORDER = 263,
    SQL_CANT_MODIFY_MASTER = 264,
    SQL_INVALID_SESSION_ID = 265,
    SQL_MAX_SESSIONS_LOGGED_IN = 266,
    SQL_MAX_DICTIONARIES_ACTIVE = 267,
    SQL_CANT_CREATE_USER_DICTIONAY = 269,
    SQL_CANT_OPEN_USER_DICTIONARY = 270,
    SQL_CANT_CREATE_USER_RIGHTS_FILE = 271,
    SQL_CANT_OPEN_USER_RIGHTS_FILE = 272,
    SQL_CANT_ACCESS_USER_RIGHTS_FILE = 273,
    SQL_INITIALIZE_OWNER_ERROR = 274,
    SQL_DUPLICATE_USER_NAME = 275,
    SQL_NOT_MASTER = 276,
    SQL_INVALID_RIGHTS = 277,
    SQL_INVALID_SUBFUNCTION_P = 278,
    SQL_INVALID_MASK = 279,
    SQL_INVALID_DICTIONARY_PATH = 280,
    SQL_ATTRIBUTE_NOT_FOUND = 281,
    SQL_INVALID_CHAR_IN_FIELD = 282,
    SQL_INVALID_RANGE_VALUE = 283,
    SQL_FIELD_VALUE_OUTSIDE_RANGE = 284,
    SQL_CANT_OPEN_DICTIONARY = 285,
    SQL_SECURITY_ALREADY_ON_OR_OFF = 286,
    SQL_NO_DICTIONARY_ACTIVE = 287,
    SQL_DICTIONARY_IN_USE = 288,
    SQL_INVALID_DICTIONARY_NAME = 289,
    SQL_DICTIONARY_ALREADY_EXISTS = 290,
    SQL_DICTIONARY_DOES_NOT_EXIST = 291,
    SQL_INVALID_MONTH = 292,
    SQL_INVALID_DAY = 293,
    SQL_INVALID_TIME = 294,
    SQL_INVALID_OPEN_MODE = 295,
    SQL_DATA_FILE_OPEN_ERROR = 296,
    SQL_FIELD_NOT_DELETABLE = 297,
    SQL_MISMATCHED_QUOTES = 298,
    SQL_INVALID_JOIN_INDEX = 299,
    SQL_MISSING_DELIMITER = 300,
    SQL_NO_RESTRICT_DEFINED = 301,
    SQL_NOT_A_USER_DEFINED_FIELD = 302,
    SQL_CANT_UPDATE_BASE_KEY = 303,
    SQL_CANT_CHANGE_PASSWORD = 304,
    SQL_VIEW_NOT_FOUND = 305,
    SQL_VIEW_STORE_ERROR = 306,
    SQL_INVALID_USER_FIELD = 307,
    SQL_ERROR_ACCESSING_DDF = 308,
    SQL_INVALID_OWNER_ACCESS = 309,
    SQL_FILE_ACCESS_DENIED = 310,
    SQL_TRANSACTIONS_NOT_ENABLED = 311,
    SQL_TRANSACTION_ABORTED = 312,
    SQL_TRANSACTIONS_ALREADY_ACTIVE = 313,
    SQL_TRANSACTIONS_NOT_ALLOWED = 314,
    SQL_DDF_CHANGE_NOT_ALLOWED = 315,
    SQL_CANNOT_UNLOAD_SQL = 316,
    SQL_DICTIONARY_LOCKED = 317,
    SQL_CANT_UPDATE_READONLY_VIEW = 318,
    SQL_INVALID_VALUE = 319,
    SQL_INVALID_CHARACTER = 320,
    SQL_INVALID_DEFAULT_VALUE = 321,
    SQL_INVALID_RANGE = 322,
    SQL_CANT_DELETE_FILE_IN_VIEW = 323,
    SQL_UNMATCHED_PARENTHESES = 324,
    SQL_INVALID_REC_LEN = 325,
    SQL_STACK_OVERFLOW_P = 326,
    SQL_NEED_NON_NULL_KEY = 327,

    SQL_INVALID_DATE_TIME_SYNTAX = 328,
    SQL_INVALID_DECIMAL_PLACES = 329,
    SQL_INVALID_DATA_FORMAT = 330,
    SQL_SECURITY_GROUP_ALREADY_EXISTS = 331,
    SQL_DDF_VERSION_TOO_OLD = 332,
    SQL_ERROR_STORING_INDEX = 333,
    SQL_DUPLICATE_INDEX_NAME = 334,
    SQL_INVALID_JOIN_TYPE = 335,
    SQL_CANT_CONVERT_VARIABLE_DATA_TYPES = 336,
    SQL_INVALID_PATHLENGTH = 337,
    SQL_INVALID_SECURITY_OPERATION = 338,
    SQL_CANT_OPERATE_ON_SELF_JOIN = 339,
    SQL_BIT_FIELD_CANT_BE_KEY = 340,
    SQL_AUTO_CANT_BE_DUP_OR_SEG = 341,
    SQL_INVALID_PARAMETER = 342,
    SQL_CANT_REMOVE_PUBLIC_USER = 343,
    SQL_NOT_VALID_ON_PUBLIC_USER = 343,
    SQL_DOS_RENAME_FAILED = 344,
    SQL_COMPUTED_FIELD_NAME_EXISTS = 345,
    SQL_MATH_OVERFLOW_ERROR = 346,
    SQL_FIELD_NAME_MISSING = 347,
    SQL_CANT_FIND_NAME_IN_CFG = 348,
    SQL_DATA_TOO_LARGE_FOR_FIELD = 349,
    SQL_SECURITY_NOT_INSTALLED = 350,
    SQL_TRANSACTION_NOT_STARTED = 351,
    SQL_INVALID_RECORD_COUNT_M = 352,
    SQL_DATA_DOESNT_MATCH_DDF = 353,
    SQL_INVALID_ICOUNT = 354,
    SQL_INVALID_CLIENT_ID_P = 355,
    SQL_MAX_CLIENTS_EXCEEDED = 356,

    SQL_UNABLE_ASSIGN_XFILE_XID = 358,
    SQL_UNABLE_ASSIGN_XFIELD_XID = 359,
    SQL_UNABLE_ASSIGN_XINDEX_XNUMBER = 360,
    SQL_INCOMPATIBLE_COL_SEQ = 361,
    SQL_OPERATION_NOT_AVAIL = 362,
    SQL_TRIGGER_DOES_NOT_EXIST = 363,
    SQL_TRIGGER_ALREADY_EXISTS = 364,
    SQL_TRIGGER_SAME_EXEC_EXISTS = 365,
    SQL_STORED_PROC_ALREADY_EXISTS = 366,
    SQL_NEED_TO_DROP_TRIGGER = 367,
    SQL_NO_FK_WITH_DC_AND_DT = 368,
    SQL_NO_MORE_USERS_GROUPS = 369,
    SQL_USER_GROUP_IN_USE = 370,
    SQL_STORED_PROC_IN_USE = 371,
    SQL_INVALID_YEAR_VALUE = 373,

    SQL_IN_DICTIONARY_NOT_ALLOWED = 380,

    SQL_INVALID_INITIAL_KEYWORD = 501,
    SQL_INVALID_SELECT = 502,
    SQL_INVALID_FUNCTIONCODE = 503,
    SQL_INVALID_OPEN_MODE_M = 504,
    SQL_INVALID_ATTRIBUTE = 505,
    SQL_INVALID_INSERT = 506,
    SQL_INVALID_UPDATE = 507,
    SQL_INVALID_DELETE = 508,
    SQL_INVALID_SET = 509,
    SQL_INVALID_CREATE = 510,
    SQL_EXPECTED_ID = 511,
    SQL_INVALID_GRANT = 512,
    SQL_MISSING_USER = 513,
    SQL_MISSING_FILE = 514,
    SQL_INVALID_ALTER = 515,
    SQL_EXPECTED_ADD_MODIFY_DROP = 516,
    SQL_MISSING_FROM = 517,
    SQL_INVALID_SORTORDER = 518,
    SQL_EXPECTED_AS = 519,
    SQL_EXPECTED_BY = 520,
    SQL_EXPECTED_VIEW_NAME = 521,
    SQL_UNKNOWN_KEYWORD = 522,
    SQL_MISSING_SECURITY_OPTION = 523,
    SQL_INVALID_SECURITY_FIELDLIST = 524,
    SQL_MUST_GIVE_PASSWORD = 525,
    SQL_NAME_CANT_BE_RESERVED = 526,
    SQL_MISSING_PASSWORD = 527,
    SQL_MISSING_CHAR = 528,
    SQL_MISSING_OWNER = 529,
    SQL_MISSING_DD_PATH = 530,
    SQL_MISSING_FILEPATH = 531,
    SQL_MISSING_EQUAL_SIGN = 532,
    SQL_INVALID_DATATYPE = 533,
    SQL_INVALID_CREATE_VIEW = 534,
    SQL_EXPECTED_INDEX_ATTRIBUTE = 535,
    SQL_EXPECTED_FIELDNAME = 536,
    SQL_FIELD_EXISTS = 537,
    SQL_INDEX_NOT_IN_FIELD_LIST = 538,
    SQL_INDEX_EXISTS = 539,
    SQL_MISSING_FIELDLIST = 540,
    SQL_MISSING_TEXT = 541,
    SQL_INVALID_KEYWORD_FOLLOWING_DROP = 542,
    SQL_MISMATCH_QUOTES = 543,
    SQL_EXPECTED_DICTIONARY_PATH = 544,
    SQL_EXPECTED_INTO = 545,
    SQL_MAX_STRINGSIZE_EXCEEDED = 546,
    SQL_MISSING_ATTRIBUTE = 547,
    SQL_INVALID_SYNTAX_FOR_TRANSACTION = 548,
    SQL_ORDER_BY_MUST_BE_LAST = 549,
    SQL_ALIAS_EXISTS = 550,
    SQL_INVALID_CHARACTERS = 551,
    SQL_EXPECTED_SELECT = 552,
    SQL_INVALID_NAMESIZE = 553,
    SQL_MISSING_GROUP = 554,
    SQL_EXPECTED_IN_DICTIONARY = 555,
    SQL_INVALID_CREATE_PARAMETER = 556,
    SQL_CANT_ACCESS_ACS_FILE = 557,
    SQL_CANT_USE_AGGR = 558,
    SQL_INVALID_SYNTAX_WITHIN_PROCEDURE = 559,
    SQL_ERROR_STORING_PROCEDURE = 560,
    SQL_STORE_STRUCT = 560,
    SQL_STORE_FILE = 560,
    SQL_STORE_VALUES = 560,
    SQL_STORE_FIELDS = 560,
    SQL_STORE_GROUPBY = 560,
    SQL_STORE_IDS = 560,
    SQL_STORE_WRITE = 560,
    SQL_STORE_COLUMN = 560,
    SQL_STORE_EXPTREE = 560,
    SQL_STORE_OPERAND = 560,
    SQL_STORE_INDEX = 560,
    SQL_STORE_SEG = 560,
    SQL_STORE_JOIN = 560,
    SQL_STORE_TABITEM = 560,
    SQL_STORE_PATHDESC = 560,
    SQL_EXPECTED_PROCDURE_NAME = 561,
    SQL_EXPECTED_PARAMETER_NAME = 562,
    SQL_DEFAULTS_NOT_ALLOWED = 563,
    SQL_PARAMETER_NOT_DEFINED = 564,
    SQL_INVALID_ISOLATION_LEV = 565,
    SQL_MISSING_ISOLATION_LEV = 566,
    SQL_INVALID_RI_SYNTAX = 567,
    SQL_CANT_REFERENCE_MORE_THAN_ONCE = 568,
    SQL_INVALID_CREATE_VIEW_SYNTAX = 569,
    SQL_INVALID_OPTION = 570,
    SQL_DUP_NAME_IN_SELECT_LIST = 571,

    SQL_MEM_M = 800,
    SQL_MAX_CUR = 801,
    SQL_COMPONENT_NOT_LOADED = 802,
    SQL_INVALID_SID_M = 803,
    SQL_INVALID_POSITION_M = 804,
    SQL_INVALID_SIZE_M = 805,
    SQL_INVALID_DEC_PLACES = 806,
    SQL_INVALID_VAR = 807,
    SQL_UNRESOLV_VARS = 808,
    SQL_INVALID_FIELD_IN_RECALLED_VIEW = 809,

    SQL_INVALID_NUM_M = 810,
    SQL_INVALID_MONTH_M = 811,
    SQL_INVALID_DAY_M = 812,
    SQL_INVALID_TIME_RANGE = 813,
    SQL_INVALID_SECONDARY_KEY = 814,
    SQL_INVALID_JOIN_FIELD = 815,
    SQL_FIELD_NOT_IN_FILE = 816,
    SQL_NOT_USED2 = 817,
    SQL_INVALID_GROUP_FIELD = 818,
    SQL_INVALID_FILE_M = 819,
    SQL_EXPECTED_COMPARISON_OP = 820,
    SQL_INVALID_CURSOR = 821,
    SQL_SHORT_BUF_M = 822,
    SQL_INVALID_SORT_INDEX = 823,
    SQL_EXPECTED_CLOSE_PARENTHESIS = 824,
    SQL_INVALID_GROUP_SUMMARY = 825,
    SQL_EXPECTED_PARENTHESIS = 826,
    SQL_GROUP_NAME_NOT_IN_LIST = 827,
    SQL_WHERE_NOT_LAST = 828,
    SQL_INVALID_COMPUTE_FIELD_SYNTAX = 829,
    SQL_EXPECTED_OPERAND = 830,
    SQL_INVALID_HAVING_SYNTAX = 831,
    SQL_INVALID_RECALL_BUF = 832,
    SQL_INVALID_VIEW_NAME = 833,
    SQL_NOT_USED3 = 834,
    SQL_INVALID_SUBFUNCTION_M = 835,
    SQL_INVALID_MASK_M = 836,
    SQL_INVALID_CREATE_DELIMITER = 837,
    SQL_INVALID_FIELD_COUNT = 838,
    SQL_INCOMPATIBLE_TYPES = 839,
    SQL_STACK_OVERFLOW_M = 840,
    SQL_CANT_CREATE_TEMP_FILE = 841,
    SQL_INVALID_CORRELATED_SUBQUERY = 842,
    SQL_MISSING_ALT_PATH = 843,
    SQL_INVALID_NUMBER_ROWS_SELECTED = 844,
    SQL_INVALID_NUMBER_OF_HEADINGS = 845,
    SQL_INVALID_SYNTAX_GROUP = 846,
    SQL_INVALID_SYNTAX_WHERE = 847,
    SQL_TOO_MANY_DISTINCTS = 848,
    SQL_RESTRICT_EXCEEDS_BUFFER_SIZE = 849,
    SQL_ERROR_CONVERTING_FILE = 850,
    SQL_CANT_ACCESS_TABLE_BEING_UPDATED = 851,
    SQL_INVALID_RECORD_COUNT = 852,
    SQL_ACCESS_DENIED_M = 853,
    SQL_NO_INDEX_DEFINED = 854,
    SQL_MAX_CURSORS_EXCEEDED = 855,
    SQL_INVALID_NUMBER_OF_VALUES = 856,
    SQL_INVALID_SYNTAX_ALL  = 857,
    SQL_INVALID_CASE_KEYWORD = 858,
    SQL_EXPECTED_FIELD_POSITION = 859,
    SQL_EXCEEDED_VIEW_BUFFER = 860,
    SQL_MISSING_VALUE = 861,
    SQL_PROCEDURE_NAME_NOT_FOUND = 862,
    SQL_PROCEDURE_HEADER_MISSING = 863,
    SQL_ERROR_RECALLING_PROCEDURE = 863,
    SQL_ERROR_READING_PROCEDURE = 863,
    SQL_PARMAMETER_NOT_DEFINED = 864,
    SQL_CANT_UPDATE_VARIABLE_LENGTH_FIELD = 865,
    SQL_CANT_USE_SUBST_ON_PROCEDURE = 866,
    SQL_INVALID_PROCEDURE_VERSION = 867,
    SQL_ERROR_ACCESSING_RI_DDF = 868,
    SQL_NO_PRIMARY_KEY_ON_PARENT_TABLE = 869,
    SQL_RI_NO_INDEX = 870,
    SQL_RI_PRIMARY_KEY_NON_UNIQUE = 871,
    SQL_RI_FOREIGN_KEY_EXISTS = 872,
    SQL_RI_PARENT_TABLE_MUST_EXIST = 873,
    SQL_RI_FOREIGN_KEY_DOESNT_EXIST = 874,
    SQL_INVALID_NULL_JOIN_ORDER = 875,
    SQL_RI_REFERENCES_EXIST = 876,
    SQL_MUST_USED_NAMED_DATABASE = 877,
    SQL_RI_TABLE_DOESNT_EXIST = 878,
    SQL_RI_KEYS_MISMATCH = 879,
    SQL_RI_KEYS_CANT_HAVE_SAME_COLUMNS = 880,
    SQL_RI_DEFINED = 881,
    SQL_RI_SELF_REFERENCE_ERROR = 882,
    SQL_RI_CYCLES_EXIST = 883,
    SQL_RI_DELETE_RULES_MUST_MATCH = 884,
    SQL_RI_MAX_LEVELS_EXCEEDED = 885,
    SQL_RI_WRONG_DATA_FILE_VERSION = 886,
    SQL_RI_FOREIGN_KEY_SAME_PARENTS = 887,
    SQL_RI_PARENT_KEY_DEFINED = 888,
    SQL_COL_NOT_IN_TABLE = 889,
    SQL_INVALID_LABEL_REF = 890,
    SQL_CONDITION_OR_REF_INVALID = 891,
    SQL_INVALID_SAVEPOINT_REF = 892,
    SQL_INVALID_CURSOR_REF = 893,
    SQL_INVALID_PARM_SYNTAX = 896,
    SQL_INVALID_CALLING_SYNTAX = 897,
    SQL_CANT_DROP_DEPENDENT_ELEMENT = 898,
    SQL_OP_NEEDS_SCROLLABLE_CURSOR = 899,
    SQL_INVALID_RECORD_NUMBER = 900,
    SQL_INVALID_STATEMENT = 902,
    SQL_INVALID_SUBJECT_TABLE_REF = 903,
    SQL_INVALID_OUTPUT_PARM = 904,
    SQL_CANT_ASSIGN_TO_CONSTANT = 905,
    SQL_CANT_EXECUTE_AGAIN = 906,
    SQL_IF_STATEMENT_INVALID = 907,
    SQL_WHILE_STATEMENT_INVALID = 908,
    SQL_FETCH_STATEMENT_INVALID = 909,
    SQL_EXCEPTION_UNDEFINED = 910,
    SQL_TRIGGER_FAILED_TO_COMPLETE = 911,
    SQL_STATEMENT_INVALID_DECLARATION = 913,
    SQL_RI_COLUMN_MORE_THAN_ONCE_IN_KEY = 914,
    SQL_COLUMN_NOT_IN_VIEW_OR_NOT_VARIABLE = 915,
    SQL_STORED_STMT_PARAM_WITHOUT_TYPE_OR_DEFAULT = 916,
    SQL_CANT_CREATE_SYSTEM_FILE_IN_PRE_6_FRMT = 917,

    /* Windows specific error codes */
    SQL_MAX_TASK_SLOTS_EXCEEDED = 1013,

    /* OS2-Specific */
    SQL_OS2_NO_MEMORY = 2001,
    SQL_OS2_BAD_PARAMETER = 2002,
    SQL_OS2_NO_LOCAL_REQUESTS = 2003,
    SQL_OS2_SPX_INSTALLATION_ERROR = 2004,
    SQL_OS2_INVALID_SPX_VERSION = 2005,
    SQL_OS2_NO_AVAILABLE_CONNECTIONS = 2006,
    SQL_OS2_INVALID_POINTER_TO_KERNEL = 2007,
    SQL_OS2_INVALID_KERNEL_VERSION = 2008,
    SQL_OS2_CANT_CREATE_CACHE_FILES = 2009,

    /* Scalable SQL Requester */
    SQL_WORKSTATION_BUFFER_TOO_SMALL = 2101,
    SQL_MAX_REDIRECTED_DEVICES_EXCEEDED = 2102,
    SQL_NOT_ACTIVE_OR_SESSION_CLOSED = 2103,
    SQL_CANT_LOGIN_MORE_THAN_ONCE = 2104,
    SQL_SERVER_BUFFER_TOO_SMALL = 2105,
    SQL_DATA_MESSAGE_BUFFER_TOO_SMALL = 2106,
    SQL_MAX_VIEWS_EXCEEDED = 2107,
    SQL_INVALID_DEVICE = 2108,
    SQL_FILE_SERVER_TABLE_TOO_SMALL = 2109,
    SQL_COULD_NOT_OPEN_IPX_SOCKET = 2110,
    SQL_RESET_FROM_SERVER_CONSOLE_DETECTED = 2111,
    SQL_SQLREQ_NOT_LOADED = 2112,
    SQL_INVALID_DRIVE_ALIAS_CFG_FILE = 2113,
    SQL_INVALID_SERVER_VOL_DRIVE_ALIAS = 2114,
    SQL_DPMI_NO_MEMORY = 2115,
    SQL_NO_SERVER_MEMORY = 2116,
    SQL_INVALID_SHARE_COUNT = 2117,
    SQL_SESSION_NOT_SHARABLE = 2118,
    SQL_INVLAID_API_FOR_SHARER = 2119,
    SQL_TASK_TABLE_FULL = 2120,
    SQL_CONN_BTW_INTF_AND_ENGINE = 2121,
    SQL_NO_THREAD_AVAIL = 2122,

    /* Miscellaneous Environment-Specific */
    SQL_CANNOT_LOGIN_MORE_THAN_ONCE = 2200,
    SQL_COULD_NOT_YIELD = 2201,
    SQL_NO_PREVIOUS_CALLBACK_ADDRESS = 2202,
    SQL_USER_CANCELLED_OPERATION = 2203,
    SQL_APPLICATION_CALLED_SQL_FROM_CALLBACK = 2204,
    SQL_INVALID_DATA = 2205,
    SQL_SEMEPHORE_ERR = 2206,
    SQL_SQL_SHUTTING_DOWN = 2208,

    /* Named Database */
    SQL_DBNM_END_OF_FILE = 2300,
    SQL_DBNM_INVALID_NAME = 2301,
    SQL_DBNM_INVALID_BUFFER_LENGTH = 2302,
    SQL_DBNM_NOT_UNIQUE = 2303,
    SQL_DBNM_INVALID_DATABASE_TYPE = 2304,
    SQL_DBNM_INVALID_DICTIONAY_PATH = 2305,
    SQL_DBNM_COULD_NOT_UPDATE_CFG = 2306,
    SQL_DBNM_COULD_NOT_OPEN_CFG = 2307,
    SQL_DBNM_INVALID_RI_FLAG = 2308,
    SQL_NAMED_DATABASE_IN_USE = 2309,

    /* Bound Database */
    SQL_DB_NOT_BOUND = 2310,
    SQL_DB_ALREADY_BOUND = 2311,
    SQL_BOUND_DB_SHARE_TABLE_ERR = 2312,
    SQL_BOUND_DB_SHARE_DICT_ERR = 2313,
    SQL_CREATE_DBNAMES_CFG_ERR = 2314,
    SQL_BIND_UNBIND_DB_ERR = 2315,
    SQL_BOUND_DB_DDF_CREATE_ERR = 2316,
    SQL_BOUND_DB_DDF_OPEN_ERR = 2317,
    SQL_BOUND_DB_NAME_CHANGE_ERR = 2318,
    SQL_BIND_UNBIND_DATAFILE_ERR = 2319,
    SQL_BIND_UNBIND_SECURED_DB_ERR = 2320,
    SQL_BOUND_DB_DDF_VERSION_ERR = 2321,
    SQL_BOUND_DB_DATAFILE_VERSION_ERR = 2322,
    SQL_DDF_NOT_FOUND_ERR = 2323,
    SQL_DDF_NOT_BOUND_ERR = 2324,
    SQL_DDF_BOUND_TO_OTHER_DB_ERR = 2325,
    SQL_DDF_BOUND_TO_UNBOUND_DB = 2326,
    SQL_DDF_BOUND_TO_UNKNOWN_DB_ERR = 2327,
    SQL_DATAFILE_NOT_FOUND_ERR = 2328,
    SQL_DATAFILE_NOT_BOUND_ERR = 2329,
    SQL_BOUND_DATAFILE_IN_UNBOUND_DB = 2330,

    /* Inscribe Engine */
    SQL_INSCRIBE_INIT_ERR = 2900,
    SQL_INSCRIBE_STOP_ERR = 2901,
    SQL_INSCRIBE_CREATE_THREAD_ERR = 2902,
    SQL_INSCRIBE_KILL_THREAD_ERR = 2903,
    SQL_INSCRIBE_LOAD_MODULE_ERR = 2904,
    SQL_INSCRIBE_UNLOAD_MODULE_ERR = 2905,
    SQL_INSCRIBE_PROC_NAME_ERR = 2906,
    SQL_INSCRIBE_PROC_EXECUTION_ERR = 2907,
    SQL_INSCRIBE_PROC_PARMS_ERR = 2908,
    SQL_INSCRIBE_ARGUMENT_BUILD_ERR = 2909,
    SQL_INSCRIBE_ARGUMENT_EXTRACT_ERR = 2910,
    SQL_INSCRIBE_LOAD_INTERPRETER_ERR = 2911,
    SQL_INSCRIBE_INTERNAL_INSTANCE_ERR = 2912,
    SQL_INSCRIBE_OUT_OF_MEMORY_ERR = 2913,
    SQL_INSCRIBE_RESOURCE_LOCK_ERR = 2914,
    SQL_INSCRIBE_DATABASE_ERR = 2915,
    SQL_INSCRIBE_SESSION_ID_ERR = 2916,
    SQL_INSCRIBE_CREDENTIAL_ERR = 2917,
    SQL_INSCRIBE_DR_CALLBACK_ERR = 2918,
    SQL_INSCRIBE_PREV_PROC_NOT_DONE_ERR = 2919,
    SQL_INSCRIBE_DISABLED_ERR = 2920,
    SQL_INSCRIBE_MODULE_ERR = 2921,
    SQL_INSCRIBE_DUP_PROCNAME_ERR = 2922
};

#define _SQLCONST_H_INCLUDED
#endif

