/****************************************************************************
**
**     Copyright (c) 1982-1997 Pervasive Software Inc. All Rights Reserved.
**
****************************************************************************/
/****************************************************************************
** VGLOBALS.C
**    This modules contains tables used by fixparms() and unfixparms().
**
****************************************************************************/

/*
**  Includes
*/
#include <sqlapi.h>
#include <fixparms.h>
#include <wxqlords.h>


BTI_WORD MaxSRBDataSize = 8192;


/*
**
**  FcnParmCount
**
**  This structure is initialized with the number of parameters each
**  function uses.  The values are ordered by function number; XQLM
**  functions are first, followed by XQLP functions, followed by DBNames
**  functions.
**
**  The parameter count does not include the XQLM cursor id or the XQLP
**  view handle since these parameters are in the fixed portion of the
**  parameter block.
*/

BTI_WORD FcnParmCount[] =
{	     /* XQLM */
     6,  0,  0,  2,  3,  6,  7,  5,  0,  0,
     2,  0,  1,  9,  4,  6,  5,
	     /* XQLP+17 */
     6,  0,  3,  0,  2,  4,  5,  5,  6,  2,
     3,  7,  4,  4,  8,  3,  5,  1,  1,  3,
         /* +20 */
    12,  3,  6,  0,  5,  5,  5, 10,  4,  3,
     2,  5,  2,  2,  8,  9,  2,  3,  1,  9,
         /* +20 */
     4,  6,  8,
         /* +2 */
         /* DBNames */
     6,  1,  1,  6,  2,  6,  2,  1,
         /* +8 */
         /* DBU* */
     7,  7
};


/*
**
**  FcnVariableLen
**
**  This structure specifies the total number of bytes of space required
**  to store the variable portion of the parameter block.  Integer
**  parameters require 2 bytes; long integer and pointer variables
**  require 4 bytes each.
**
*/


BTI_WORD FcnVariableLen[] =
{
    /* XQLM */
    22,  0,   0,  6,  8, 16, 16, 14,
    0,  0,   6,  0,  4, 24, 12, 14,
    12,

    /* XQLP */
    22,  0,  10,  0,  4, 10, 12, 16,
    16,  6,  10, 22, 14, 14, 26, 14,
    16,  4,   2,  8, 38, 10, 18,  0,
    14, 14,  14, 30, 12,  8,  6, 14,
    6,   6,  24, 28,  8,  6,  4, 24,
    12, 14,  22,

    /* DBNames */
    24,  4,   4, 24,  8, 24,  8,   2,

    /* DBU* */
    24, 24
};


/*
**  initialized at runtime to starting offset of each function's
**  offset into tables
*/

BTI_WORD initParmOffsets = TRUE;


/*
**
**  parmOffsets
**
**  This structure is initialized at runtime with a cumulative offset
**  using the values in FcnParmCount.
**
**  SRBFormat/SRBUnformat/fixParms/unfixParms use these values to
**  determine the starting offset into the parmLength and parmFlags
**  arrays for each function.
**
*/

BTI_WORD ParmOffsets[MAX_SSQL_FUNCTIONS] = {0};


/*
**  parmLength
**
**  This array contains the maximum size of each string parameter for
**  each function call.  Zeros are used as place holders for integer and
**  long integer parameters.  If the parameter is a buffer of variable
**  length, the constant CVAR is used to indicate this.  The data buffer
**  length or count parameter is used to determine the total buffer size.
**
**  Again, these lengths are not specified for the XQLM cursorid or
**  the XQLP view handle.
**
*/

BTI_BYTE parmLength[] = {
							/*  XQLLogin	 */
    C30, C8, C64, C64, C8, 0,
							/*  XQLCompile	 */
    0, CVAR,
							/*  XQLFormat	 */
    0, 0, CVAR,
							/*  XQLFetch	 */
    0, 0, CVAR,  0, 0, 0,
							/*  XQLDescribe  */
    0, 0, 0, 0, 0, 0, CVAR,
							/*  XQLSubst	 */
    0, 0, CVAR, 0, CVAR,
							/*  XQLStatus	 */
    0, C30,
							/*  XQLVersion	 */
    C4,
							/*  XQLConvert	 */
    0, 0, 0, 0, 0, CVAR, CVAR, C30, 0,
							/*  XQLValidate  */
    0, C44, 0, CVAR,
							/*  XQLMask      */
    0, 0, 0, 0, 0, CVAR,
							/*  XQLSPUtility */
    0, 0, 0, 0, CVAR,
							/*    xLogin	 */
    C30, C8, C64, C64, C8, 0,
							/*    xNew  	 */
    C20, C8, 0,
							/*    xMovefld	 */
    0, 0,
							/*    xField	 */
    0, 0, 0, C44,
							/*    xDescribe  */
    0, 0, 0, 0, CVAR,
							/*    xFetch	 */
    0, 0, 0, 0, CVAR,
							/*    xCompute	 */
    C20, 0, 0, 0, 0, CVAR,
							/*    xOrder	 */
    0, C46,
							/*    xRestrict  */
    0, 0, CVAR,
							/*    xJoin      */
    C20, C8, 0, 0, C44, 0, C44,
							/*    xInsert	 */
    0, C20, 0, CVAR,
							/*    xUpdate	 */
    0, C20, 0, CVAR,
							/*    xUpdall	 */
    0, C20, 0, 0, 0, 0, C44, C44,
							/*    xRemove	 */
    0, C20, 0,
							/*    xRemall	 */
    0, C20, 0, 0, 0,
							/*    xReset	 */
    C8,
							/*    xTrans	 */
    0,
							/*    xStatus	 */
    0, 0, CVAR,
							/*    xDDCreate  */
    0, C20, 0, C64, C8, 0, 0, C28, 0, C22, 0, CVAR,
							/*    xStore	 */
    C20, 0, CVAR,
							/*    xRecall	 */
    C20, 0, C8, 0, 0, CVAR,
							/*    xDDFile	 */
    0, 0, C20, 0, CVAR,
							/*    xDDField	 */
    0, 0, C44, 0, CVAR,
							/*    xDDIndex	 */
    0, 0, C44, 0, CVAR,
							/*    xDDModify  */
    0, C20, 0, C64, C8, 0, 0, C28, 0, C22,
							/*    xDDView	 */
    0, C20, 0, CVAR,
							/*    xDDDrop	 */
    C20, 0, 0,
							/*    xDDPath	 */
    0, C64,
							/*    xDDAttr	 */
    0, C44, 0, 0, CVAR,
							/*    xDD        */
    C64, 0,
							/*    xSecurity  */
    C8, 0,
							/*    xUser      */
    C8, 0, C30, C8, 0, 0, 0, CVAR,
							/*    xAccess	 */
    C8, C30, 0, 0, C20, 0, C44, 0, CVAR,
							/*    xPassword  */
    C30, C8,
							/*    xChar      */
    0, 0, 0,
							/*    xVersion	 */
    C4,
							/*    xConvert	 */
    0, 0, 0, 0, 0, CVAR, CVAR, C30, 0,
							/*    xValidate  */
    0, C44, 0, CVAR,
							/*    xMask      */
    0, 0, 0, 0, 0, CVAR,
							/*    xVersion2  */
    C4, 0, 0, CVAR, 0, 0, 0, C4,
                                                   /* SQLAddDataBaseName */
    C20, C10, 0, CVAR, 0, CVAR,
                                                  /* SQLDropDatabaseName */
    C20,
                                             /* SQLGetCountDatabaseNames */
    0,
                                                   /* SQLGetDatabaseInfo */
    C20, C10, 0, CVAR, 0, CVAR,
                                                  /* SQLGetDatabaseNames */
    0, CVAR,
                                                /* SQLModifyDatabaseInfo */
    C20, C10, 0, CVAR, 0, CVAR,
                                                    /* SQLRenameDatabase */
    C20, C20,
                                                     /* SQLUnloadDBNames */
    0,
                                                           /* DBUGetInfo */
    0, 0, 0, 0, CVAR, 0, CVAR,
                                                           /* DBUSetInfo */
    0, 0, 0, 0, CVAR, 0, 0
};


/*
**
**  parmFlags
**
**  For each parameter, this array contains flags that indicate the
**  parameter type.  The flags are defined in 'fixparms.h'.
**
**  Again, these lengths are not specified for the XQLM cursorid or the
**  XQLP view handle.
**
*/

BTI_WORD parmFlags[] =
{
							/* XQLLogin */
    STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA+REDIR_DEVICE,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* XQLCompile */
    CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      STR_PARM+SEND_DATA,

							/* XQLFormat */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      STR_PARM+SEND_DATA,

							/* XQLFetch */
    CONST_VAL+SEND_DATA,
      CONST_VAL+LEN_VAL+SEND_DATA+RET_DATA,
      DATA_PARM+RET_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* XQLDescribe */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      STR_PARM+RET_DATA+SPEC_PARM,

							/* XQLSubst */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA,

							/* XQLStatus */
    CONST_VAL+SEND_DATA,
      DATA_PARM+RET_DATA+SPEC_PARM,

							/* XQLVersion */
    DATA_PARM+RET_DATA,

							/* XQLConvert */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      DATA_PARM+SEND_DATA+SPEC_PARM,
      DATA_PARM+RET_DATA+SPEC_PARM,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* XQLValidate */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA,

							/* XQLMask */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      STR_PARM+SEND_DATA+RET_DATA,

							/* XQLSPUtility  */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+RET_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA+RET_DATA,

                            /* xLogin */
    STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA+REDIR_DEVICE,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* xNew */
    STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* xMovefld */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* xField */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+CNT_VAL+SEND_DATA+RET_DATA,
      STR_PARM+SEND_DATA,

							/* xDescribe */
    CONST_VAL+SEND_DATA,
      CONST_VAL+LEN_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+CNT_VAL+SEND_DATA+RET_DATA,
      DATA_PARM+RET_DATA+SEND_DATA+SPEC_PARM,

							/* xFetch */
    CONST_VAL+LEN_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,
      DATA_PARM+SEND_DATA+RET_DATA+SPEC_PARM,

							/* xCompute */
    STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+LEN_VAL+SEND_DATA+RET_DATA,
      STR_PARM+SEND_DATA,

							/* xOrder */
    CONST_VAL+CNT_VAL+SEND_DATA,
      DATA_PARM+SEND_DATA,

							/* xRestrict */
    CONST_VAL+SEND_DATA,
      CONST_VAL+LEN_VAL+SEND_DATA+RET_DATA,
      STR_PARM+SEND_DATA,

							/* xJoin */
    STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,

							/* xInsert */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA+SPEC_PARM+RET_DATA,

							/* xUpdate */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA+SPEC_PARM,

							/* xUpdall */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,
      CONST_VAL+SEND_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,

							/* xRemove */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,

							/* xRemall */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,
      LONG_VAL+SEND_DATA+RET_DATA,

							/* xReset */
    STR_PARM+SEND_DATA,

							/* xTrans */
    CONST_VAL+SEND_DATA,

							/* xStatus */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA,

							/* xDDCreate */
    CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA+SPEC_PARM,
      CONST_VAL+SEND_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA,

							/* xStore */
    STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA+SPEC_PARM,

							/* xRecall */
    STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA,

							/* xDDFile */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA+RET_DATA+SPEC_PARM,

							/* xDDField */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA,

							/* xDDIndex */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA,

							/* xDDModify */
    CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA+SPEC_PARM,
      CONST_VAL+SEND_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA,

							/* xDDView */
    CONST_VAL+SEND_DATA+RET_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA,

							/* xDDDrop */
    STR_PARM+SEND_DATA+SPEC_PARM,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* xDDPath */
    CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA+REDIR_DEVICE,

							/* xDDAttr */
    CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA+RET_DATA+SPEC_PARM,

							/* xDD */
    STR_PARM+SEND_DATA+REDIR_DEVICE,
      CONST_VAL+SEND_DATA,

							/* xSecurity */
    STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* xUser */
    STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA+RET_DATA+SPEC_PARM,

							/* xAccess */
    STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      DATA_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA+SPEC_PARM,

							/* xPassword */
    STR_PARM+SEND_DATA,
      STR_PARM+SEND_DATA,

							/* xChar */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA,

							/* xVersion */
    DATA_PARM+RET_DATA,

							/* xConvert */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      DATA_PARM+SEND_DATA+SPEC_PARM,
      DATA_PARM+RET_DATA+SPEC_PARM,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA,

							/* xValidate */
    CONST_VAL+SEND_DATA+RET_DATA+CNT_VAL,
      STR_PARM+SEND_DATA,
      CONST_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA,

							/* xMask */
    CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      STR_PARM+SEND_DATA+RET_DATA,

							/* xVersion2 */
    LONG_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA,
      SPEC_PARM+DATA_PARM+SEND_DATA,
      CONST_VAL+RET_DATA,
      CONST_VAL+RET_DATA,
      CONST_VAL+RET_DATA,
      LONG_VAL+RET_DATA,

                                               /* SQLAddDataBaseName */
    DATA_PARM+SEND_DATA,
      DATA_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+LEN_VAL,
      STR_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+LEN_VAL,
      STR_PARM+SEND_DATA,

                                              /* SQLDropDatabaseName */
    DATA_PARM+SEND_DATA,

                                         /* SQLGetCountDatabaseNames */
    LONG_VAL+RET_DATA,

                                               /* SQLGetDatabaseInfo */
    DATA_PARM+SEND_DATA,
      DATA_PARM+RET_DATA,
      LONG_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      STR_PARM+RET_DATA,
      LONG_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      STR_PARM+RET_DATA,

                                              /* SQLGetDatabaseNames */
    LONG_VAL+LEN_VAL+SEND_DATA+RET_DATA,
      DATA_PARM+SEND_DATA+RET_DATA,

                                            /* SQLModifyDatabaseInfo */
    DATA_PARM+SEND_DATA,
      DATA_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+LEN_VAL,
      STR_PARM+SEND_DATA,
      LONG_VAL+SEND_DATA+LEN_VAL,
      STR_PARM+SEND_DATA,

                                                /* SQLRenameDatabase */
    DATA_PARM+SEND_DATA,
      DATA_PARM+SEND_DATA,

                                                 /* SQLUnloadDBNames */
    CONST_VAL+SEND_DATA+RET_DATA,

                                                       /* DBUGetInfo */
    LONG_VAL+SEND_DATA, CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA, LONG_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA, LONG_VAL+SEND_DATA+RET_DATA+LEN_VAL,
      DATA_PARM+RET_DATA,

                                                       /* DBUSetInfo */
    LONG_VAL+SEND_DATA, CONST_VAL+SEND_DATA,
      CONST_VAL+SEND_DATA, LONG_VAL+SEND_DATA+LEN_VAL,
      DATA_PARM+SEND_DATA, LONG_VAL+SEND_DATA, LONG_VAL+SEND_DATA
};


