/****************************************************************************
**
** Copyright (c) 1982-1997 Pervasive Software Inc. All Rights Reserved.
**
****************************************************************************/
/****************************************************************************
** WXQLORDS.H
**    This module contains the SSQL function numbers for use by fixparms.c
**
****************************************************************************/
#if !defined(_WXQLORDS_H)
#define _WXQLORDS_H


#define MAX_XQLM_FUNCTIONS  17
#define MAX_XQLP_FUNCTIONS  43
#define MAX_DBN_FUNCTIONS   8
#define MAX_SSQL_FUNCTIONS  MAX_XQLM_FUNCTIONS \
        + MAX_XQLP_FUNCTIONS + MAX_DBN_FUNCTIONS


/*
**  note that the following SQL-Level and Relation Primitive Function
**  Numbers *must* be used in conjunction with the signature of the
**  parameter block ("XQLP" || "XQLM") since they overlap 0-16.
*/

/* Relational Primitives Function Numbers */
#define pLogInFcn    0
#define pLogOutFcn   1
#define pNewFcn      2
#define pFreeFcn     3
#define pMoveFldFcn  4
#define pFieldFcn    5
#define pDescribeFcn 6
#define pFetchFcn    7
#define pComputeFcn  8
#define pOrderFcn    9
#define pRestrictFcn 10
#define pJoinFcn     11
#define pInsertFcn   12
#define pUpdateFcn   13
#define pUpdallFcn   14
#define pRemoveFcn   15
#define pRemallFcn   16
#define pResetFcn    17
#define pTransFcn    18
#define pStatFcn     19
#define pDDCreFcn    20
#define pStoreFcn    21
#define pRecallFcn   22
#define pStopFcn     23
#define pDDFileFcn   24
#define pDDFieldFcn  25
#define pDDIndexFcn  26
#define pDDModFcn    27
#define pDDViewFcn   28
#define pDDDrpFcn    29
#define pDDPathFcn   30
#define pDDAttrFcn   31
#define pDDFcn       32
#define pSecurFcn    33
#define pUserFcn     34
#define pAccessFcn   35
#define pPswdFcn     36
#define pCharFcn     37
#define pVersionFcn  38
#define pCnvFcn      39
#define pValFcn      40
#define pMaskFcn     41
#define pVers2Fcn    42


/* SQL-Level Function Numbers */
#define mLogInFcn     0
#define mLogOutFcn    1
#define mCursorFcn    2
#define mCompFcn      3
#define mFormatFcn    4
#define mFetchFcn     5
#define mDescribeFcn  6
#define mSubstFcn     7
#define mExecFcn      8
#define mFreeFcn      9
#define mStatFcn      10
#define mStopFcn      11
#define mVersFcn      12
#define mConvFcn      13
#define mValFcn       14
#define mMaskFcn      15
#define mSPUtilFcn    16

/* DBNames Function Numbers */
#define dbnAdd        0
#define dbnDrop       1
#define dbnGetCnt     2
#define dbnGetInfo    3
#define dbnGetNames   4
#define dbnModify     5
#define dbnRename     6
#define dbnUnload     7


/*
**  Sequenced Function Numbers.  Note that these numbers are used
**  *internally* by the engine and don't refer to the interfaces nor do
**  they require that we check the signature of the parameter block.
*/

/*
**  these are used *internally* as when using the table driven
**  paramblock reformatter (fixparms).
*/

#define smLogInFcn      0
#define smLogOutFcn     1
#define smCursorFcn     2
#define smCompFcn       3
#define smFormatFcn     4
#define smFetchFcn      5
#define smDescribeFcn	6
#define smSubstFcn      7
#define smExecFcn       8
#define smFreeFcn       9
#define smStatFcn       10
#define smStopFcn       11
#define smVersFcn       12
#define smCnvFcn        13
#define smValFcn        14
#define smMaskFcn       15
#define smSPUtilFcn     16

#define spLogInFcn      17
#define spLogOutFcn     18
#define spNewFcn        19
#define spFreeFcn       20
#define spMoveFldFcn    21
#define spFieldFcn      22
#define spDescribeFcn   23
#define spFetchFcn      24
#define spCompFcn       25
#define spOrderFcn      26
#define spRestrictFcn   27
#define spJoinFcn       28
#define spInsertFcn     29
#define spUpdateFcn     30
#define spUpdallFcn     31
#define spRemoveFcn     32
#define spRemallFcn     33
#define spResetFcn      34
#define spTransFcn      35
#define spStatFcn       36
#define spDDCreFcn      37
#define spStoreFcn      38
#define spRecallFcn     39
#define spStopFcn       40
#define spDDFileFcn     41
#define spDDFieldFcn    42
#define spDDIndexFcn    43
#define spDDModFcn      44
#define spDDViewFcn     45
#define spDDDrpFcn      46
#define spDDPathFcn     47
#define spDDAttrFcn     48
#define spDDFcn         49
#define spSecurFcn      50
#define spUserFcn       51
#define spAccessFcn     52
#define spPswdFcn       53
#define spCharFcn       54
#define spVersFcn       55
#define spCnvFcn        56
#define spValFcn        57
#define spMaskFcn       58
#define spVers2Fcn      59

/*
**  for now, these are used by the DOS client only, not the NLM or
**  Windows clients.  Still, they need to be placed in the sequenced list
**  so the variables in vglobals.c don't get out of whack.
*/

#define sdbnAdd         60
#define sdbnDrop        61
#define sdbnGetCnt      62
#define sdbnGetInfo     63
#define sdbnGetNames    64
#define sdbnModify      65
#define sdbnRename      66
#define sdbnUnload      67

/*
**  these are used as ordinal #s for MS Windows DLLs.
*/

#define sWNSQLINITFcnOrd       1
#define sXQLPCALLFcnOrd        2
#define spComputeFcnOrd        3
#define spDescribeFcnOrd       4
#define spMoveFldFcnOrd        5
#define spFetchFcnOrd          6
#define spFieldFcnOrd          7
#define spFreeFcnOrd           8
#define spEditFcnOrd           9
#define spInsertFcnOrd         10
#define spJoinFcnOrd           11
#define spLogInFcnOrd          12
#define spLogOutFcnOrd         13
#define spNewFcnOrd            14
#define spOrderFcnOrd          15
#define spEditallFcnOrd        16
#define spRemallFcnOrd         17
#define spRemoveFcnOrd         18
#define spResetFcnOrd          19
#define spRestrictFcnOrd       20
#define spStopFcnOrd           21
#define spTransFcnOrd          22
#define spStoreFcnOrd          23
#define spRecallFcnOrd         24
#define spUpdateFcnOrd         25
#define spUpdallFcnOrd         26
#define spDDFcnOrd             27
#define spDDAttrFcnOrd         28
#define spDDCreFcnOrd          29
#define spDDModFcnOrd          30
#define spDDDrpFcnOrd          31
#define spDDFieldFcnOrd        32
#define spDDFileFcnOrd         33
#define spDDIndexFcnOrd        34
#define spDDPathFcnOrd         35
#define spDDViewFcnOrd         36
#define spAccessFcnOrd         37
#define spPswdFcnOrd           38
#define spSecurFcnOrd          39
#define spUserFcnOrd           40
#define spCharFcnOrd           41
#define spVersionFcnOrd        42
#define spStatFcnOrd           43
#define spCnvFcnOrd            44
#define spValFcnOrd            45
#define spMaskFcnOrd           46

#define sXQLMCallFcnOrd        47
#define smCompFcnOrd           48
#define smCursorFcnOrd         49
#define smDescribeFcnOrd       50
#define smExecFcnOrd           51
#define smFetchFcnOrd          52
#define smFormatFcnOrd         53
#define smFreeFcnOrd           54
#define smLogInFcnOrd          55
#define smLogOutFcnOrd         56
#define smSPUtilFcnOrd         57
#define smStatFcnOrd           58
#define smStopFcnOrd           59
#define smSubstFcnOrd          60
#define smVersFcnOrd           61
#define smCnvFcnOrd            62
#define smValFcnOrd            63
#define smMaskFcnOrd           64
#define sWEPFcnOrd             65
#define spGetSessionFcnOrd     66
#define spPutSessionFcnOrd     67
#define spShareSessionIDFcnOrd 68
#define smCallbackFcnOrd       69

/*
**  Since DBNames support in Windows is a stand-alone DLL, we don't need
**  to include the ordinals in this range.
*/

#endif /* _WXQLORDS_H */
